"""
life-simplifiyng stuff for lazy devs
--------
"""



__title__ = "team4958_customs"
__author__ = "arti404"
__license__ = "MIT"
__copyright__ = "Copyright 2023-present arti404"
__version__ = "0.0.0"



from team4958_customs import tools, utilities
from team4958_customs.utils import MISSING, ROOT



__all__ = [
    'tools',
    'utilities',
    'MISSING',
    'ROOT'
]